package com.keyware.shandan.bianmu.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.keyware.shandan.bianmu.entity.DirectoryResource;
import com.keyware.shandan.bianmu.entity.DirectoryVo;
import com.keyware.shandan.bianmu.enums.DirectoryType;
import com.keyware.shandan.bianmu.enums.ReviewStatus;
import com.keyware.shandan.bianmu.mapper.DirectoryResourceMapper;
import com.keyware.shandan.bianmu.utils.DirectoryUtil;
import com.keyware.shandan.common.constants.DirConstant;
import com.keyware.shandan.common.entity.TreeVo;
import com.keyware.shandan.common.util.StreamUtil;
import com.keyware.shandan.common.util.StringUtils;
import com.keyware.shandan.common.util.TreeUtil;
import com.keyware.shandan.frame.config.security.SecurityUtil;
import com.keyware.shandan.system.entity.SysUser;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.stream.Collectors;

/**
 * @author Administrator
 */
@Service
public class DirectoryTreeService {
    private final DirectoryService directoryService;
    private final DirPermissionService dirPermissionService;
    private final DirectoryResourceMapper directoryResourceMapper;


    public DirectoryTreeService(DirectoryService directoryService, DirPermissionService dirPermissionService, DirectoryResourceMapper directoryResourceMapper) {
        this.directoryService = directoryService;
        this.dirPermissionService = dirPermissionService;
        this.directoryResourceMapper = directoryResourceMapper;
    }

    /**
     * 获取目录树
     *
     * @param parentId     父级目录ID
     * @param reviewStatus 审核状态
     * @param hasMetadata  是否包含数据资源
     * @param permis       权限类型
     * @return 目录树
     */
    public List<TreeVo> tree(String parentId, String reviewStatus, boolean hasMetadata, String permis, boolean allChild, int busType) {
        final String pid = StringUtils.hasText(parentId) ? parentId : "";
        DirectoryVo parentDir = null;
        if (StringUtils.hasText(pid)) {
            parentDir = directoryService.getById(pid);
        }
        List<DirectoryVo> directoryList = getDirectoryList(parentDir, allChild, hasMetadata, busType);
        ReviewStatus status = StringUtils.hasText(reviewStatus) ? ReviewStatus.valueOf(reviewStatus) : null;
        SysUser user = SecurityUtil.getLoginSysUser();
        Set<String> permisDis = "write".equals(permis) ? dirPermissionService.getByWritePermis(user) : dirPermissionService.getByReadPermis(user);
        List<DirectoryVo> permisDirectoryList = StreamUtil.as(directoryList).filter(dir -> {
            if (status == null) {
                return permisDis.contains(dir.getId());
            }
            return dir.getReviewStatus() == status && permisDis.contains(dir.getId());
        }).toList();
        DirectoryParentBuilder dpb = new DirectoryParentBuilder(directoryList, permisDirectoryList);

        // 转换为Dtree对象
        List<TreeVo> treeVoList = StreamUtil.as(dpb.build().get()).map(DirectoryUtil::dir2Tree).toList();

        // 构建树形结构
        treeVoList = StreamUtil.as(TreeUtil.buildDirTree(treeVoList, pid))
                // 过滤根节点的垃圾数据
                .filter(dir -> (dir.getParentId().equals(pid)))
                .toList();

        return treeVoList.stream().filter(tree -> !tree.getId().equals("ROOT_THEME")).collect(Collectors.toList());
    }

    /**
     * 获取目录的列表
     *
     * @param parentDir   父级目录
     * @param allChild    是否查询所有下级
     * @param hasResource 是否包含数据资源
     * @return 目录集合
     */
    private List<DirectoryVo> getDirectoryList(DirectoryVo parentDir, boolean allChild, boolean hasResource, int busType) {
        QueryWrapper<DirectoryVo> wrapper = new QueryWrapper<>();
        if(parentDir == null){
            wrapper.eq("ID", "ROOT");
        }else {
            wrapper.in(!hasResource, "DIRECTORY_TYPE", DirectoryType.DIRECTORY, DirectoryType.LINK_DIR).eq("IS_BASIC_DIR", busType);
            if (parentDir != null) {
                if (allChild) {
                    wrapper.likeRight("DIRECTORY_PATH", parentDir.getDirectoryPath() + "/");
                } else {
                    wrapper.eq("PARENT_ID", parentDir.getId());
                }
            }
        }

        return directoryService.list(wrapper);
    }

    public Page<DirectoryResource> directoryResourcePage(Page<DirectoryResource> page, DirectoryResource directoryResource) {
        QueryWrapper<DirectoryResource> wrapper = new QueryWrapper<>(directoryResource);
        Set<String> allowDirIds = dirPermissionService.getByReadPermis(SecurityUtil.getLoginSysUser());
        //
        if (StringUtils.hasText(directoryResource.getResourceName())) {
            wrapper.and(queryWrapper -> {
                queryWrapper.
                        like("RESOURCE_NAME", directoryResource.getResourceName())
                        .or()
                        .like("RESOURCE_COMMENT", directoryResource.getResourceName());
            });
            directoryResource.setResourceName(null);
        }
        if (directoryResource.getDirectoryPath() != null) {
            String path = directoryResource.getDirectoryPath();
            if (path.endsWith("/")) {
                path = path.substring(0, path.length() - 1);
            }
            DirectoryVo parent = directoryService.getByPath(path);
            List<DirectoryVo> dirList = directoryService.childrenLists(parent);
            HashSet<String> ids = (HashSet<String>) dirList.stream().filter(child -> allowDirIds.contains(child.getId())).map(DirectoryVo::getId).collect(Collectors.toSet());
            ids.add(parent.getId());
            wrapper.in("PARENT_ID", ids);
            directoryResource.setDirectoryPath(null);
        }
        return directoryResourceMapper.selectPage(page, wrapper);
    }

    private static class DirectoryParentBuilder {
        private final Map<String, DirectoryVo> allDirMap = new HashMap<>();
        private final Map<String, DirectoryVo> targetMap = new HashMap<>();

        private final ArrayBlockingQueue<DirectoryVo> queue;

        public DirectoryParentBuilder(List<DirectoryVo> allDirList, @NonNull List<DirectoryVo> list) {
            allDirList.forEach(dir -> allDirMap.put(dir.getId(), dir));
            list.forEach(item -> targetMap.put(item.getId(), item));
            queue = new ArrayBlockingQueue<>(Math.max(allDirList.size(), 1));
            queue.addAll(this.targetMap.values());
        }

        public DirectoryParentBuilder build() {
            while (queue.size() > 0) {
                try {
                    DirectoryVo dir = queue.take();
                    DirectoryVo parent = targetMap.get(dir.getParentId());
                    if (parent == null) {
                        parent = allDirMap.get(dir.getParentId());
                        if (parent != null) {
                            queue.put(parent);
                            targetMap.put(parent.getId(), parent);
                        }
                    }
                } catch (InterruptedException ignored) {
                }
            }
            // 主题目录
            DirectoryVo themeRoot = allDirMap.get(DirConstant.DIR_THEME_ROOT_ID);
            if (targetMap.get(DirConstant.DIR_THEME_ROOT_ID) == null && themeRoot != null) {
                targetMap.put(themeRoot.getId(), themeRoot);
            }
            return this;
        }

        public List<DirectoryVo> get() {
            return StreamUtil.as(targetMap.values()).sorted(Comparator.comparing(DirectoryVo::getCreateTime)).toList();
        }
    }
}
