package com.keyware.shandan.datasource.utils;

import com.keyware.shandan.datasource.enums.ConnType;
import org.springframework.util.FileCopyUtils;
import org.springframework.util.StringUtils;

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.sql.Driver;
import java.util.*;
import java.util.jar.Attributes;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.Manifest;
import java.util.stream.Stream;

/**
 * <p>
 * Jar包动态加载器
 * </p>
 *
 * @author GuoXin
 * @since 2021-05-19
 */
public class DriverJarLoader {

    private static URLClassLoader classLoader;

    private static final String DEFAULT_DRIVER_TYPE = "java.sql.Driver";

    private static void initUrlClassLoader(){
        if(classLoader == null){
            classLoader = (URLClassLoader) ClassLoader.getSystemClassLoader();
        }
    }


    /**
     * 功能描述: 扫描一个文件夹下面的所有jar，不包含子文件夹和子jar
     *
     * @param directoryPath 要扫描的路径
     * @return -
     * @throws NoSuchMethodException      -
     * @throws InvocationTargetException-
     * @throws IllegalAccessException-
     * @throws IOException-
     */
    public static Map<String, Class<?>> loadAllJarFromAbsolute(String directoryPath) throws
            NoSuchMethodException, InvocationTargetException, IllegalAccessException, IOException {

        File directory = new File(directoryPath);
        // 判断是否为文件夹，如果是文件,直接用单个jar解析的方法去解析
        if (!directory.isDirectory()) {
            // 添加jar扫描路径
            addUrl(directory);
            return loadJarFromAbsolute(directoryPath);
        }
        // 如果是文件夹，则需要循环加载当前文件夹下面的所有jar
        Map<String, Class<?>> clazzMap = new HashMap<>(16);
        File[] jars = directory.listFiles();
        if (jars != null && jars.length > 0) {
            List<String> jarPath = new LinkedList<>();
            for (File file : jars) {
                String fPath = file.getPath();
                // 只加载jar
                if (fPath.endsWith(".jar")) {
                    addUrl(file);
                    jarPath.add(fPath);
                }
            }
            if (jarPath.size() > 0) {
                for (String path : jarPath) {
                    clazzMap.putAll(loadJarFromAbsolute(path));
                }
            }
        }
        return clazzMap;
    }

    /**
     * 功能描述: 从绝对路径中加载jar包中的类
     * * 扫描指定jar包前需要将这个jar的地址加入了系统类加载器的扫描列表中
     * * 注意，这里只支持单个jar的加载，如果这个jar还引入了其他jar依赖，会加载失败
     * * 所以只能用来加载对其他jar包没有依赖的简单对象类信息
     *
     * @param path jar包路径
     * @return -
     * @throws IOException -
     */
    public static Map<String, Class<?>> loadJarFromAbsolute(String path) throws IOException {
        JarFile jar = new JarFile(path);
        Enumeration<JarEntry> entryEnumeration = jar.entries();
        Map<String, Class<?>> clazzMap = new HashMap<>(16);
        while (entryEnumeration.hasMoreElements()) {
            JarEntry entry = entryEnumeration.nextElement();
            // 先获取类的名称，符合条件之后再做处理，避免处理不符合条件的类
            String clazzName = entry.getName();
            if (clazzName.endsWith(".class")) {
                // 去掉文件名的后缀
                clazzName = clazzName.substring(0, clazzName.length() - 6);
                // 替换分隔符
                clazzName = clazzName.replace("/", ".");
                // 加载类,如果失败直接跳过
                try {
                    Class<?> clazz = Class.forName(clazzName);
                    // 将类名称作为键，类Class对象作为值存入mao
                    // 因为类名存在重复的可能，所以这里的类名是带包名的
                    clazzMap.put(clazzName, clazz);
                } catch (Throwable e) {
                    e.printStackTrace();
                }
            }
        }
        return clazzMap;
    }


    /**
     * 功能描述: 添加需要扫描的jar包
     *
     * @param jarPath jar包路径
     * @throws NoSuchMethodException     -
     * @throws InvocationTargetException -
     * @throws IllegalAccessException    -
     * @throws MalformedURLException     -
     */
    public static void addUrl(File jarPath) throws NoSuchMethodException, InvocationTargetException,
            IllegalAccessException, MalformedURLException {
        initUrlClassLoader();
        // 反射获取类加载器中的addURL方法，并将需要加载类的jar路径
        Method method = URLClassLoader.class.getDeclaredMethod("addURL", URL.class);
        if (!method.isAccessible()) {
            method.setAccessible(true);
        }
        URL url = jarPath.toURI().toURL();
        // 把当前jar的路径加入到类加载器需要扫描的路径
        method.invoke(classLoader, url);

    }

    /**
     * 获取jar包版本
     *
     * @param path jar包路径
     * @return 版本号
     * @throws IOException -
     */
    public static String jarVersion(String path) throws IOException {
        JarFile jar = new JarFile(path);
        Manifest manifest = jar.getManifest();
        String version = "";
        if (manifest != null) {
            Attributes attributes = manifest.getMainAttributes();
            if (attributes != null) {
                //Implementation-Version
                version = attributes.getValue("Implementation-Version");
            }
        }
        return version == null ? "" : version;
    }

    public static String findDriverClassName(String jarPath, ConnType desc) throws IOException {
        String className = "";
        JarFile jarFile = new JarFile(jarPath);
        JarEntry entry = jarFile.getJarEntry(getSpiDesc(desc));
        if (entry != null) {
            StringWriter writer = new StringWriter();
            FileCopyUtils.copy(new InputStreamReader(jarFile.getInputStream(entry)), writer);
            jarFile.close();
            StringReader reader = new StringReader(writer.toString());
            LineNumberReader lineNumberReader = new LineNumberReader(reader);
            Stream<String> stream = lineNumberReader.lines();
            className = stream.filter(line -> !line.startsWith("#")).findFirst().orElse("");
            lineNumberReader.close();
        }else {
            throw new RuntimeException("驱动程序验证失败：没有检测到java.sql.Driver描述");
        }

        if(!StringUtils.hasText(className)){
            Map<String, Class<?>> classMap = loadJarFromAbsolute(jarPath);
            className = classMap.values().stream().filter(c -> {
                Class<?>[] interfaces = c.getInterfaces();
                return Arrays.stream(interfaces).anyMatch(clazz -> clazz.getTypeName().equals(DEFAULT_DRIVER_TYPE));
            }).map(Class::getTypeName).findFirst().orElse("");
        }
        return className;
    }

    /**
     * 获取SPI描述文件名
     * @param desc -
     * @return -
     */
    private static String getSpiDesc(ConnType desc){
        return "META-INF/services/" + desc.getSpiDesc();
    }

    public static Driver getDriverInstance(String className){
        try {
            Class<?> clazz = classLoader.loadClass(className);
            return (Driver)clazz.newInstance();
        } catch (ClassNotFoundException | InstantiationException | IllegalAccessException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 关闭类加载器，释放jar文件
     */
    public static void clear() {
        try {
            if(classLoader != null){
                classLoader.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
